package cn.com.qmth.examcloud.config.center.core;

import cn.com.qmth.examcloud.commons.logging.ExamCloudLog;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLogFactory;
import cn.com.qmth.examcloud.commons.util.MapUtil;
import cn.com.qmth.examcloud.commons.util.RegExpUtil;
import cn.com.qmth.examcloud.commons.util.StringUtil;
import org.apache.commons.lang3.StringUtils;

import java.util.Map;
import java.util.Map.Entry;

/**
 * 命令解释器
 *
 * @author WANGWEI
 * @date 2018年12月5日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public class CommandInterpreter {

    private static ExamCloudLog log = ExamCloudLogFactory.getLog("COMMAND_LOGGER");

    private static CommandInterpreter interpreter;

    private static final Object LOCK = new Object();

    private static long sessionTimeout = 1000 * 60 * 10;

    /**
     * 构造函数
     */
    private CommandInterpreter() {

    }

    /**
     * 获取单例
     */
    public static CommandInterpreter getInstance() {
        if (null == interpreter) {
            synchronized (LOCK) {
                if (null == interpreter) {
                    interpreter = new CommandInterpreter();
                }
                return interpreter;
            }
        } else {
            return interpreter;
        }
    }

    /**
     * 解释
     */
    public void interpret(String cmd) {
        try {
            process(cmd);
        } catch (Exception e) {
            log.error(e.getMessage());
        }
    }

    /**
     * 处理
     */
    public void process(String cmd) {
        cmd = cmd.trim();
        if (StringUtils.isBlank(cmd)) {
            return;
        }

        if (cmd.matches("\\s*session\\s+[1-9]\\d*")) {
            // 设置进程退出命令：session 分钟值
            String timeoutStr = RegExpUtil.find(cmd, "\\s*session\\s+([1-9]\\d*)", 1);
            timeoutStr = timeoutStr.length() > 4 ? "999" : timeoutStr;

            // 默认最大值
            long timeout = StringUtil.toLong(timeoutStr);
            timeout = timeout > 360 ? 360 : timeout;

            sessionTimeout = 1000 * 60 * timeout;
            log.info("sessionTimeout = " + timeout + " minutes");
        } else if (cmd.matches("\\s*load\\s+([^\\s]+)\\s*")) {
            // 加载配置:load <appSimpleName>
            String appSimpleName = RegExpUtil.find(cmd, "\\s*load\\s+([^\\s]+)\\s*", 1);
            String active = BootstrapSecurityManager.getInstance().getActive();
            Map<String, String> properties = PropertiesLoader.getProperties(appSimpleName, active);

            properties = MapUtil.sortMapByKey(properties, true);
            StringBuilder props = new StringBuilder()
                    .append(appSimpleName)
                    .append(" ")
                    .append(active)
                    .append(" Properties: ");

            for (Entry<String, String> entry : properties.entrySet()) {
                props.append("\n  ").append(entry.getKey()).append("=").append(entry.getValue());
            }

            log.info(props.toString());
        } else {
            // 错误命令
            log.error("error command !");
        }

    }

    public long getSessionTimeout() {
        return sessionTimeout;
    }

}
